#include "sRs232.h"

//-----------------------------------------------------------------------------
bool sRs232::Configurer(int Vitesse,int NbBits,int Parite)
{
	int Res;
	//Ouverture de la com.
	//fd = open(pPort,O_RDWR | O_NONBLOCK);
	fd = open(port.c_str(), O_RDWR | O_NOCTTY); //SeA
	if (fd == -1) {
		message = " Erreur Ouverture " + port;
   		return false;
	}
	
	tcgetattr(fd, &Config);
	
	// Suppression de l'echo
	Config.c_lflag &= ~(ECHO) ;
	
	Config.c_iflag |= IGNBRK | IGNPAR;
	Config.c_oflag = 0;
	
	// Ajout SeA //////////////////////////////////////////
	Config.c_cflag |= (CLOCAL | CREAD);   // Enable the receiver and set local mode
	Config.c_cflag &= ~CSTOPB;            // 1 stop bit
	Config.c_cflag &= ~CRTSCTS;           // Disable hardware flow control
	Config.c_lflag &= ~ICANON; 			  // Enlever le mode canonique
										  // Mode canonique activé : on attent le RETURN pour sortir du read
										  // Mode canonique désactivé : le read fonctionne normalement
	
	cfmakeraw(&Config); 				  // Mode Raw : lecture car par car
	Config.c_cc[VMIN]  = 1;				  // Au moins 1 caractère
	Config.c_cc[VTIME] = 0;				  // Attente infinie
	//////////////////////////////////////////////////////
	
	// Vitesse
	switch (Vitesse)
	{
		case BPS9600: 	Config.c_cflag |= B9600; cfsetspeed(&Config, B9600); break;
		case BPS4800: 	Config.c_cflag |= B4800; cfsetspeed(&Config, B4800); break;
		case BPS2400: 	Config.c_cflag |= B2400; cfsetspeed(&Config, B2400); break;
		case BPS19200: 	Config.c_cflag |= B19200; cfsetspeed(&Config, B19200); break;
		default: 		message = " Valeur BAUD incorrecte";return false;
	}
	
	// Taille data
	switch (NbBits)
	{
		case BIT8: 		Config.c_cflag |= CS8;break;
		case BIT7: 		Config.c_cflag |= CS7;break;
		case BIT6: 		Config.c_cflag |= CS6;break;
		case BIT5: 		Config.c_cflag |= CS5;break;
		default: 		message = " Valeur BIT incorrecte"; return false;
	}

	// Parite
	switch (Parite)
	{
		case PARITE_PAIRE : 	Config.c_cflag |=  PARENB; break;
		case PARITE_IMPAIRE : 	Config.c_cflag |= (PARODD | PARENB);break;
		default : 				break;
	}
	
	// Application de la configuration
	
	Res = tcsetattr(fd, TCSANOW, &Config);
	if (Res == -1)
	{
		message = " Erreur application paramètres";
		return false;
	}
	tcflush(fd, TCIFLUSH);
	tcflush(fd, TCOFLUSH);
	return true;
}

//----------------------------------------------------------------------------
string sRs232::getError()
{
	return message;
}

 
//----------------------------------------------------------------------------
sRs232::sRs232(string pPort):port(pPort)
{   
}

//----------------------------------------------------------------------------
sRs232::sRs232(void)
{
   port = COM1;
}

//----------------------------------------------------------------------------
sRs232::~sRs232()
{
   close(fd);
}

//----------------------------------------------------------------------------
int sRs232::Envoyer(string pChaine)
{
   int Res = write (fd, pChaine.c_str(), pChaine.size());
   return (Res);
}

//----------------------------------------------------------------------------
// Recevoir Nb caractères, attente infinie
string sRs232::Recevoir(int Nb)
{
	char Car;
	int Res;
	string msg = "";
	
	do
	{
		Res = read (fd, &Car, 1);
		if (Res > 0)
			msg += Car;
	} 
	while (msg.size() < Nb);
	
	return msg;
}

//----------------------------------------------------------------------------
string sRs232::Recevoir(char Fin, int Mode)
// Recevoir des caractères jusqu'au caractère Fin, attente infinie
{
	char Car = 0x00;
	int Res;
	string msg = "";

	do
	{
		Res = read(fd, &Car, 1);
		if (Res > 0)
			msg += Car;
	} 
	while (Car != Fin);
	
	if (Mode == SANS_FINAL)
		//msg.pop_back(); // C++11
		msg.erase(msg.size() - 1, 1);
		
	return msg;
}

//----------------------------------------------------------------------------
string sRs232::Recevoir(char Fin, int Mode, int Attente) // Attente en ms
// Recevoir des caractères jusqu'au caractère Fin, sortie sur delai d'attente
{
	char Car = 0x00;
	int Res = 1;
	string msg;
	
	// Modification du VTIME pour le delai d'attente du caractère:
	termios Config_old;
	tcgetattr(fd, &Config);  // initialisation de la structure de réglage
	tcgetattr(fd, &Config_old); // Sauvegarde de la config du port série
	Config.c_cc[VTIME] = Attente / 100; // temps en 1/10 s
	Config.c_cc[VMIN] = 0;
	tcsetattr(fd, TCSANOW, &Config);

	do
	{
		Res = read(fd, &Car, 1);
		if (Res > 0)
			msg += Car;
	} 
	while (Car != Fin && Res != 0);
	
	 if (Mode == SANS_FINAL && Res != 0 )
		 msg.erase(msg.size() - 1, 1);
		
	tcsetattr(fd, TCSANOW, &Config_old); // Remise en place du réglage d'origine
	return msg;
}

//----------------------------------------------------------------------------
string sRs232::RecevoirCaracteres(int Attente) // Attente en ms
// Recevoir des caractères,  sortie sur delai d'attente
{
	return Recevoir(0x01, AVEC_FINAL, Attente);
}

//----------------------------------------------------------------------------
bool sRs232::RecevoirFichier(string NomFichier, int AttenteDebut)
{
	FILE *Fichier;
	char Car;
	char chCar[2];
	int AttenteFin = 2000;
	string data = "";
	
	data = RecevoirCaracteres(AttenteDebut);	
	
	if ( data.size() > 0)
	{
		if ( (Fichier = fopen(NomFichier.c_str(), "w")) == NULL) {
			message = " Erreur ouverture fichier";
			return false;
		}
		
		fputs ( data.c_str(), Fichier);
		fclose(Fichier);
		return true;
	}
	message = " Pas de data...";
	return false;
}
