#!/usr/bin/env node
var WebSocketServer = require('websocket').server;
var http = require('http');

// Creation du Socket serveur http de base
var server = http.createServer(function(request, response) {
    console.log((new Date()) + ' Received request for ' + request.url);
    response.writeHead(404);
    response.end();
});

// Serveur en ecoute. Seul le port est défini. Protocole http://.
server.listen(8080, function() {
    console.log((new Date()) + ' Server is listening on port 8080');
});

// Mise en place du WebSocket (protocole ws://) à partir du socket serveur
wsServer = new WebSocketServer({
    httpServer: server,
    // You should not use autoAcceptConnections for production
    // applications, as it defeats all standard cross-origin protection
    // facilities built into the protocol and the browser.  You should
    // *always* verify the connection's origin and decide whether or not
    // to accept it.
    autoAcceptConnections: false
});

function originIsAllowed(origin) {
  // put logic here to detect whether the specified origin is allowed.
  return true;
}

////////////////////////////////////////////////////
// SeA : Essai d'envoi d'un message commun à tous les clients connectés
var listeConnexions = []; // Liste des clients
i = 0;
setInterval(envoiCyclique, 3000);
////////////////
// ANALYSE:
// Lancé par le setInterval
// La variable i reste commune au processus.
// On ne la modifie qu'une fois à chaque passage 
function envoiCyclique() { 
	listeConnexions.forEach( (client, index) => {
		if (index == 0) i++;
		let port = client.socket._peername.port;
		client.sendUTF(client.remoteAddress + ":" + port + " i:" + i);
		//console.log('envoi à ', client.remoteAddress, ' ', port);
	});
}

//// Mettre à jour le tableau des connexions
function supprimer(conn) {
	let portC = conn.socket._peername.port;
	let adresseC = conn.remoteAddress;
	
	listeConnexions.forEach( (client, index) => {
		let port = client.socket._peername.port;
		let adresse = client.remoteAddress;
		if ( adresse == adresseC && port == portC ) 
			listeConnexions.splice(index, 1);
	});
}
/////////////////////////////////////////////////////


// Si le serveur est contacté en mode ws://
wsServer.on('request', function(request) {
    if (!originIsAllowed(request.origin)) {
      // Make sure we only accept requests from an allowed origin
      request.reject();
      console.log((new Date()) + ' Connection from origin ' + request.origin + ' rejected.');
      return;
    }
    
	// SeA: On accepte une connexion avec l'option 'echo-protocole'
	// 'echo-protocol' est une option que le client doit utiliser à la connexion
	// Exemple : socket = new WebSocket("ws://192.168.1.79:8080",'echo-protocol');
	// NB :
	// le ACCEPT ne génère pas un nouveau thread (voir # ps au -T).
	//	Il s'insère dans une boucle d'évènements
    var connection = request.accept('echo-protocol', request.origin);
    console.log((new Date()) + ' Connection accepted. IP:', connection.remoteAddress, ' Port:', connection.socket._peername.port);
	
	// Pour chaque message reçu:
    connection.on('message', function(message) {
        if (message.type === 'utf8') {
            console.log('Received Message: ' + message.utf8Data);
			// Envoi du message au client:
            connection.sendUTF(message.utf8Data);
        }
        else if (message.type === 'binary') {
			// Envoi du message au client:
            console.log('Received Binary Message of ' + message.binaryData.length + ' bytes');
            connection.sendBytes(message.binaryData);
        }
    });
	
    connection.on('close', function(reasonCode, description) {
        console.log((new Date()) + ' Peer ' + connection.remoteAddress + ' disconnected.');
		// SeA :
		supprimer(connection);
    });
	
	//SeA : Gestion des connexions multiples
	listeConnexions.push(connection);
});

